/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef CATEGORYITEMWIDGETHEADERFILE
#define CATEGORYITEMWIDGETHEADERFILE

#include <QWidget>
#include <QString>
#include <QStringList>

class QRadioButton;
class QLabel;
class QComboBox;
class QLineEdit;
class QCheckBox;

class TComponentItemWidget : public QWidget
{
Q_OBJECT
public:
enum TRadioOptions
{
	CONSTANT_LINEAR=0,
	CONSTANT_TEMPERATURE,
	LENGTH_DEPENDENT_LINEAR,
	LENGTH_DEPENDENT_TEMPERATURE
};
	
// CREATORS
	TComponentItemWidget(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	~TComponentItemWidget(void);
	
// ACCESSORS
	TRadioOptions Radio_Options(void) const;
	QString UCType(void) const;
	QString Distribution_Type(void) const;
	double Sensitivity(void) const;
	QString Expression(void) const;
	bool Use_Default_Expression(void) const;
	
// MANIPULATORS	
	void Reset_Distribution_Types(const QStringList &list);
	void Reset_Uncertainty_Types(const QStringList &list);
	
	void Set_Idenfitier_Name(const QString &identifier);
	void Set_Radio_Options(const TRadioOptions options);	
	void Set_UCType(const QString &type);	// Type A or Type B
	void Set_Distribution_Type(const QString &type);
	void Set_Sensitivity(const double &val);
	void Set_Expression(const QString &text);
	void Set_Option_Use_Default(const bool state);
	void Enable_Option_Use_Default(const bool state);
	
signals:
	void Component_Item_Changed(void);

private slots:
	void Use_Default_Toggled(bool);
	void Expression_Changed(void);
	void Item_Modified(void);
	
private:
	
	QLabel								*d_ucitem_label;
	QRadioButton						*d_linear_radio;
	QRadioButton						*d_temperature_radio;
	QRadioButton						*d_constant_radio;
	QRadioButton						*d_length_dependent_radio;
	QLabel								*d_expression_description_label;
	QComboBox							*d_distribution_combo;
	QLineEdit							*d_sensitivity_edit;
	QLineEdit							*d_expression_edit;
	QCheckBox							*d_use_default_check;
	QComboBox							*d_uctype_combo;
	
	bool								d_block_modifed_signal;

// NOT IMPLEMENTED
	TComponentItemWidget(const TComponentItemWidget&);
	TComponentItemWidget& operator=(const TComponentItemWidget&);
};

#endif
