/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef CATEGORYWIDGETHEADERFILE
#define CATEGORYWIDGETHEADERFILE

#include <QWidget>
#include <QString>
#include <QStringList>

class QRadioButton;
class QDateEdit;
class QCheckBox;
class QComboBox;
class QLabel;
class QListWidget;
class QLineEdit;
class QToolButton;
class TMessageBox;

class TComponentWidget : public QWidget
{
Q_OBJECT
public:
	
// CREATORS
	TComponentWidget(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	~TComponentWidget(void);
	
// ACCESSORS
	QString Current_Identifier(void) const;
	QString Calibration_Date(void) const;
	QString Calibration_Due_Date(void) const;
	double Expansion_Coefficient(void) const;
	bool Mode_Single(void) const;
	bool Mode_Sum(void) const;
	bool Mode_Export_Equipment(void) const;
	bool Mode_Limit_Uncertainty_Length(void) const;
	double Length(void) const;
		
// MANIPULATORS	
	void Set_Identifiers(const QStringList &ids);
	
	void Set_Dates(const QString &cal_date,const QString &cal_due_date);
	void Clear_Dates(void);	// greys the dates
	void Set_Expansion_Coefficient(const double &value);
	void Set_Mode_Single(void);
	void Set_Mode_Sum(void);
	void Set_Mode_Export_Equipment(const bool state);
	void Set_Mode_Limit_Uncertainty_Length(const bool state);
	void Set_Length(const double &len);
	
signals:
	void Add_Component_Identifier(const QString&);
	void Remove_Component_Identifiers(const QStringList&);
	void Rename_Component_Identifiers(const QString&,const QString&);			// old_name, new_name
	
	void Component_Changed(void);
	void Component_Identifier_Selected(const QString&);
	
private slots:
	void Add_Identifier(void);
	void Remove_Identifiers(void);
	void Rename_Identifier(QWidget *editor);
	void Identifier_Selected(void);

	void Data_Changed(void);
		
private:
	
	QRadioButton						*d_ucmode_individual_radio;
	QRadioButton						*d_ucmode_combine_radio;
	QCheckBox							*d_export_equipment_check;
	QCheckBox							*d_limit_length_check;
	QDateEdit							*d_calibration_date;
	QDateEdit							*d_calibration_due_date;
	QLineEdit							*d_expansion_coefficient_edit;
	QListWidget							*d_identifiers_list;
	QLineEdit							*d_length_edit;
	QToolButton							*d_add_identifier_button;
	QToolButton							*d_remove_identifiers_button;
	TMessageBox							*d_msg_box;

	QStringList							d_current_identifier_list;
	QString								d_current_identifier;
	bool								d_bock_modifed_signal;
			
// NOT IMPLEMENTED
	TComponentWidget(const TComponentWidget&);
	TComponentWidget& operator=(const TComponentWidget&);
};

#endif
