/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#include <QVariant>
#include <QAction>
#include <QApplication>
#include <QButtonGroup>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QHeaderView>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>

#include "namedialog.h"

TNameDialog::TNameDialog(
	const QWidget						*parent,
	Qt::WindowFlags						flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*dialog_layout;
	QLabel								*name_label;
	QHBoxLayout							*button_hlayout;
	QSpacerItem							*button_hspacer;
	QPushButton							*cancel_button;

	this->resize(315, 100);
	dialog_layout = new QGridLayout(this);

	name_label = new QLabel(this);
	dialog_layout->addWidget(name_label, 0, 0, 1, 1);

	d_name_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_name_edit, 0, 1, 1, 1);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	cancel_button = new QPushButton(this);
	cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(cancel_button);

	d_ok_button = new QPushButton(this);
	d_ok_button->setAutoDefault(false);
	d_ok_button->setDefault(true);
	d_ok_button->setEnabled(false);
	button_hlayout->addWidget(d_ok_button);

	dialog_layout->addLayout(button_hlayout, 1, 0, 1, 2);
		
	name_label->setText(QStringLiteral("Name"));
	cancel_button->setText(QStringLiteral("Cancel"));
	d_ok_button->setText(QStringLiteral("Ok"));
	
	connect(d_ok_button,&QPushButton::clicked,this,&TNameDialog::accept);
	connect(cancel_button,&QPushButton::clicked,this,&TNameDialog::reject);
	connect(d_name_edit,&QLineEdit::textChanged,this,&TNameDialog::Text_Changed);
}

TNameDialog::~TNameDialog(void)
{
}

QString TNameDialog::Name(void) const
{
	return d_name_edit->text().trimmed();
}

void TNameDialog::Set_Name(
	const QString						&name)
{
	d_name_edit->setText(name);
}

void TNameDialog::Set_Title(
	const QString						&title)
{
	this->setWindowTitle(title);
}

void TNameDialog::Text_Changed(
	const QString						&text)
{
	d_ok_button->setEnabled(text.length() != 0);
}
