/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef REPORTGROUPWIDGETHEADERFILE
#define REPORTGROUPWIDGETHEADERFILE

#include <QWidget>
#include <QString>
#include <QStringList>
#include <vector>

#include "../share/types.h"

class QListWidget;
class QListWidgetItem;
class QToolButton;

class TReportGroupWidget : public QWidget
{
Q_OBJECT
public:
	
// CREATORS
	TReportGroupWidget(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	~TReportGroupWidget(void);
	
// ACCESSORS
	int Group_Count(void) const {return static_cast<int>(d_groups.size());}
	TGroup Group(const int index) const;
	std::vector<TGroup> Groups(void) const {return d_groups;}
	QStringList Group_Names(void) const;
	QStringList Group_List(const QString &group_name) const;
	
// MANIPULATORS
	void Update_Reportable_Components(const QStringList &list);
	void Set_Groups(const std::vector<TGroup> &groups);
	
	void Clear_Groups(void);
	bool Add_Group(const TGroup &group);
	
signals:
	void Report_Groups_Changed(void);
	
private slots:
	void Uc_Item_Selection_Changed(void);	
	void Report_Group_Selection_Changed(void);
	void Create_New_Group(void);
	void Remove_Selected_Groups(void);
	void Rename_Current_Group(QWidget *editor);

private:
	
	QListWidget							*d_uccomponents_list;
	QListWidget							*d_reportgroups_list;
	QToolButton							*d_add_reportgroup_button;
	QToolButton							*d_remove_reportgroup_button;
	
	QString								d_current_report_group;
	std::vector<TGroup>					d_groups;
	
	QListWidgetItem* Find_Component_Item(const QString &name) const;
	
	void Remove_Group(const QString &name);
	void Rename_Group(const QString &old_name, const QString &new_name);
	QStringList Group_Items(const QString &name);
	void Modify_Group(const QString &group_name,const QStringList &items);

// NOT IMPLEMENTED
	TReportGroupWidget(const TReportGroupWidget&);
	TReportGroupWidget& operator=(const TReportGroupWidget&);
};

#endif
