/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef UCBUDGETHEADERFILE
#define UCBUDGETHEADERFILE

#include <QString>
#include <QStringList>

#include "../share/types.h"
#include "../share/ucbudgetdata.h"

#include "../../core/expression.h"

#include <vector>

class TXmlFile;

class TUCBudget
{
public:
	
// CREATORS
	TUCBudget(void);
	~TUCBudget(void);
	
// ACCESSORS
	QStringList Enum_Components(void) const;
	QStringList Enum_Component_Items(const QString &component_name) const;
	QStringList Enum_Component_Identifiers(const QString &component_name) const;
	QStringList Enum_Reportable_Components(void) const;
	
	bool Component_Item_Default_Expression(const QString &component_name,
								 const QString &component_item_name,
								 QString *default_expression) const;
	bool Component_Item_Expression(const QString &component_name,
										 const QString &component_item_name,
										 const QString &identifier,
										 bool *use_default,
										 QString *replacement_expression) const;
	QString Component_Item_Expression(const QString &component_name,const QString &component_item_name,const QString &identifier,bool *valid = 0) const;
	
	double Component_Length(const QString &component_name,const QString &identifier) const;
	double Component_Expansion_Coefficient(const QString &component_name,const QString &identifier) const;
	
	bool Convert_From_Reportable(const QString &name,QString *component_name,QString *component_item_name) const;
	
	QStringList Distribution_Type_List(void) const;
	QStringList Uncertainty_Type_List(void) const;

	// conversion functions
	QString Distribution_Type(const TUCBudgetData::TDistribution type) const;
	TUCBudgetData::TDistribution Distribution_Type(const QString &type) const;
	QString UCType(const TUCBudgetData::TUCType type) const;
	TUCBudgetData::TUCType UCType(const QString &type) const;
	TUCBudgetData::TUCMode UCMode(const QString &mode) const;
	QString UCMode(const TUCBudgetData::TUCMode mode) const;
	QString Expression_Type(const TUCBudgetData::TExpressionType type) const;
	TUCBudgetData::TExpressionType Expression_Type(const QString &type) const;
	bool Boolean_Type(const QString &yes_no) const;
	QString Boolean_Type(const bool yes_no) const;
	
	double Distribution_Divisor(const TUCBudgetData::TDistribution type) const;
	
	TUCBudgetData* Budget_Data(void) {return &d_budget_data;}
	
	bool Save(const QString &file_name,
			  const std::vector<TVariable> &variables,
			  const std::vector<TGroup> &groups,
			  const TInputData &input_data) const;
	
	QString Get_Last_Error(void) const {return d_last_error;}

// MANIPULATORS
		
	bool Update_Component_Item(const QString &component,
							   const QString &component_item,
							   const QString &identifier,
							   const QString &expression,
							   const bool use_default);
	
	bool Load(const QString &file_name,
			  std::vector<TVariable> *variables,
			  std::vector<TGroup> *groups,
			  TInputData *input_data);
	
	void Reset_Expression_Variables(void);
	void Add_Expression_Variable(const QString &name,const double &value);
	
	double Evaluate_Reportable_Component(const QString &description,const QString &identifier,const double &length,bool *valid);
	double Evaluate_Component_Item(const QString &component_name,const QString &component_item_name,const QString &identifier,const double &length,bool *valid);
		
private:

	TUCBudgetData						d_budget_data;
	TExpression							d_expression;
	
	mutable QString						d_last_error;
	
	void LoadV1(TXmlFile *xml_file,
				std::vector<TVariable> *variables,
				std::vector<TGroup> *groups,
				TInputData *input_data);
	
	void LoadV2(TXmlFile *xml_file,
				std::vector<TVariable> *variables,
				std::vector<TGroup> *groups,
				TInputData *input_data);
	
	void LoadV3(TXmlFile *xml_file,
				std::vector<TVariable> *variables,
				std::vector<TGroup> *groups,
				TInputData *input_data);

// NOT IMPLEMENTED
	TUCBudget(const TUCBudget&);
	TUCBudget& operator=(const TUCBudget&);
};



#endif
