/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef UCBUDGETEDITORHEADERFILE
#define UCBUDGETEDITORHEADERFILE

#include <QMainWindow>
#include <QDir>

#include "ucbudget.h"

class QTabWidget;
class QTreeWidget;
class QTreeWidgetItem;
class QWidget;
class QAction;
class QMenuBar;
class QPlainTextEdit;
class QToolButton;

class TComponentWidget;
class TComponentItemWidget;
class TVariableWidget;
class TReportGroupWidget;
class TReportWidget;
class TMessageBox;

class QCloseEvent;
class QDragEnterEvent;
class QDropEvent;

class TUCBudgetEditor : public QMainWindow
{
Q_OBJECT
	
public:
// CREATORS
	TUCBudgetEditor(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TUCBudgetEditor(void);
	
// ACCESSORS
	
// MANIPULATORS
	
private slots:
	void BudgetTree_Selection_Changed(void);
	void Tab_Selection_Changed(int);
	
	void BudgetTree_Rename_Item(QWidget *editor);
	void BudgetTree_Add_Item(void);
	void BudgetTree_Remove_Item(void);

	void Component_Identifier_Selected(const QString &identifier_name);
	void Component_Add_Identifier(const QString &name);
	void Component_Rename_Identifier(const QString &old_name, const QString &new_name);
	void Component_Remove_Identifiers(const QStringList &names);
	void Component_Data_Changed(void);
	
	void Component_Item_Data_Changed(void);
	void Variable_Data_Chanaged(void);
	void Report_Groups_Changed(void);
	
	void Update_Report_View(const QString &group_name);
	void Generate_Export_Report(const QString &group_name);
		
	void New(void);
	void Open(void);
	void Save(void);
	void SaveAs(void);
	void Exit(void);
		
	void View_Budget_Data(void);
	void View_Variables(void);
	void View_Groups(void);
	void View_Report(void);
	void View_Log(void);
	
protected:
	virtual void closeEvent(QCloseEvent *event);
	virtual void dragEnterEvent(QDragEnterEvent *DragEnterEvent);
	virtual void dropEvent(QDropEvent *DropEvent);

private:
	
struct TReportData
{
	QString								component_name;
	QString								component_item_name;
	QString								identifier;
	double								length;
	
	std::vector<TVariable>				variables;
};
	
	QTabWidget							*d_tab_widget;
    QTreeWidget							*d_budget_tree;
	
	TComponentWidget					*d_component_widget;
	TComponentItemWidget				*d_component_item_widget;
	TVariableWidget						*d_variable_widget;
	TReportGroupWidget					*d_reportgroup_widget;
	TReportWidget						*d_report_widget;
	QPlainTextEdit						*d_log_edit;
	
	TMessageBox							*d_msg_box;
	
	QAction 							*d_new_action;
	QAction 							*d_open_action;
	QAction 							*d_save_action;
	QAction 							*d_saveas_action;
	QAction								*d_exit_action;
	
	QAction								*d_add_budgetitem_action;
	QAction								*d_remove_budgetitem_action;
	QToolButton							*d_add_budgetitem_button;
	QToolButton							*d_remove_budgetitem_button;

	QAction 							*d_components_action;
	QAction 							*d_variables_action;
	QAction 							*d_groups_action;
	QAction 							*d_report_action;
	QAction 							*d_log_action;
	
	bool								d_data_modified;
	QString								d_save_file_name;
	QString								d_data_path;
	
	QString								d_current_component_name;
	QString								d_current_component_identifier_name;
	QString								d_current_component_item_name;
	QString								d_default_component_identifier_name;
	int									d_current_item_index;

	TUCBudget							d_ucbudget;
	
	std::vector<TReportData>			d_reporting_data;
	std::vector<TReportData>			d_reporting_variable_data;
	std::vector<TVariable>				d_base_variables;
	
	void Load_Data(const QString &file_name);
	void Create_Report_Data(const QString &group_name);
	bool Update_Report(const double &length);
	bool Update_Export(const double &length);
	QStringList Report_Data_Variable_List(void);
	QStringList Variable_List(const QString &component_name,const QString &identifier);
	QStringList Variable_List(const QString &component_name,const QString &component_item_name,const QString &identifier) const;
	void Remove_From_List(QStringList *list,const QString &item) const;
	
	void Write_Log_Text(const QString &t1,
						const QString &t2=QString(),
						const QString &t3=QString(),
						const QString &t4=QString(),
						const QString &t5=QString(),
						const QString &t6=QString(),
						const QString &t7=QString(),
						const QString &t8=QString());
	
	void Update_BudgetTree(void);
	void Select_BudgetTree_Item(const QString &component = QString(),const QString &component_item = QString());

	
// NOT IMPLEMENTED
	TUCBudgetEditor(const TUCBudgetEditor&);
	TUCBudgetEditor& operator=(const TUCBudgetEditor&);
};


#endif

