/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef VARIABLEWIDGETHEADERFILE
#define VARIABLEWIDGETHEADERFILE

#include <QWidget>
#include <QString>
#include <QStringList>
#include <vector>

#include "../share/types.h"

class QLineEdit;
class QToolButton;
class TOptionEditorWidget;

class TVariableWidget : public QWidget
{
Q_OBJECT
public:
	
// CREATORS
	TVariableWidget(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	~TVariableWidget(void);
	
// ACCESSORS
	TInputData Input_Data(void) const;

	bool Variable_Exists(const QString &name) const;
	
	int Variable_Count(void) const {return static_cast<int>(d_variables.size());}
	int Static_Variable_Count(void) const {return d_static_variable_count;}
	TVariable Variable(const int index) const;
	QString Variable_Name(const int index) const;
	double Variable_Value(const int index,bool *valid) const;
	QString Variable_Description(const QString &variable_name) const;
	
	double Start_Position(void) const;
	double End_Position(void) const;
	double Increment(void) const;
	double Machine_Expansion_Coefficient(void) const;

	
// MANIPULATORS
	void Update_Variable_List(const QStringList &list);
	void Add_Optional_Variables(const std::vector<TVariable> &variables);
	void Set_Input_Data(const TInputData &input_data);
	
private slots:
	void Variable_Selection_Changed(void);
	void Variable_Selection_List_Changed(const int,const QString&);

	void Add_Variable(void);
	void Rename_Variable(void);
	void Remove_Variable(void);
	void Variable_Value_Changed(void);

signals:
	void Variable_Changed(void);
	
private:
	
	QLineEdit							*d_machine_expansion_edit;
	QLineEdit							*d_machine_scale_edit;
	QLineEdit							*d_minimum_temperature_edit;
	QLineEdit							*d_maximum_temperature_edit;
	QLineEdit							*d_gradient_temperature_edit;
	QLineEdit							*d_mean_stdev_edit;
	QLineEdit							*d_start_position_edit;
	QLineEdit							*d_end_position_edit;
	QLineEdit							*d_increment_edit;
	QToolButton							*d_add_variable_button;
	QToolButton							*d_rename_variable_button;
	QToolButton							*d_remove_variable_button;
	TOptionEditorWidget					*d_variable_table;
	
	int									d_static_variable_count;
	QStringList							d_variable_list;
	
	std::vector<TVariable>				d_variables;
	
	
	void Rename_Variable_Entry(const QString &old_name,const QString &new_name);
	void Remove_Variable_Entry(const QString &name);
	void Initialize_Option_Editor(void);
	void Update_Option_Editor(void);
	
// NOT IMPLEMENTED
	TVariableWidget(const TVariableWidget&);
	TVariableWidget& operator=(const TVariableWidget&);
};

#endif
